@@ -85,6 +85,7 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
   g_autoptr(FlatpakContext) app_context = NULL;
   gboolean custom_usr;
   g_auto(GStrv) runtime_ref_parts = NULL;
+  FlatpakRunFlags run_flags;
 
   context = g_option_context_new (_("DIRECTORY [COMMAND [args...]] - Build in directory"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
@@ -116,8 +117,12 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
     command = argv[rest_argv_start + 1];
 
   app_deploy = g_file_new_for_commandline_arg (directory);
-
   metadata = g_file_get_child (app_deploy, "metadata");
+
+  if (!g_file_query_exists (app_deploy, NULL) ||
+      !g_file_query_exists (metadata, NULL))
+    return flatpak_fail (error, _("Build directory %s not initialized, use flatpak build-init"), directory);
+
   if (!g_file_load_contents (metadata, cancellable, &metadata_contents, &metadata_size, NULL, error))
     return FALSE;
 
@@ -171,9 +176,17 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
             "--bind", flatpak_file_get_path_cached (app_files), "/app",
             NULL);
 
+  if (!custom_usr)
+    add_args (argv_array,
+              "--lock-file", "/usr/.ref",
+              NULL);
+
+  run_flags = FLATPAK_RUN_FLAG_DEVEL | FLATPAK_RUN_FLAG_NO_SESSION_HELPER;
+  if (custom_usr)
+    run_flags |= FLATPAK_RUN_FLAG_WRITABLE_ETC;
+
   if (!flatpak_run_setup_base_argv (argv_array, NULL, runtime_files, NULL, runtime_ref_parts[2],
-                                    FLATPAK_RUN_FLAG_DEVEL | FLATPAK_RUN_FLAG_NO_SESSION_HELPER,
-                                    error))
+                                    run_flags, error))
     return FALSE;
 
   /* After setup_base to avoid conflicts with /var symlinks */
