@@ -66,6 +66,9 @@ flatpak_builtin_document_list (int argc, char **argv,
   else
     app_id = argv[1];
 
+  if (argc > 2)
+    return usage_error (context, _("Too many arguments"), error);
+
   session_bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, error);
   if (session_bus == NULL)
     return FALSE;
@@ -115,7 +118,7 @@ flatpak_complete_document_list (FlatpakCompletion *completion)
       user_dir = flatpak_dir_get_user ();
       {
         g_auto(GStrv) refs = flatpak_dir_find_installed_refs (user_dir, NULL, NULL, NULL,
-                                                              TRUE, FALSE, &error);
+                                                              FLATPAK_KINDS_APP, &error);
         if (refs == NULL)
           flatpak_completion_debug ("find local refs error: %s", error->message);
         for (i = 0; refs != NULL && refs[i] != NULL; i++)
@@ -126,10 +129,10 @@ flatpak_complete_document_list (FlatpakCompletion *completion)
           }
       }
 
-      system_dir = flatpak_dir_get_system ();
+      system_dir = flatpak_dir_get_system_default ();
       {
         g_auto(GStrv) refs = flatpak_dir_find_installed_refs (system_dir, NULL, NULL, NULL,
-                                                              TRUE, FALSE, &error);
+                                                              FLATPAK_KINDS_APP, &error);
         if (refs == NULL)
           flatpak_completion_debug ("find local refs error: %s", error->message);
         for (i = 0; refs != NULL && refs[i] != NULL; i++)
