@@ -48,6 +48,10 @@ struct BuilderManifest
   char           *sdk;
   char           *sdk_commit;
   char           *var;
+  char           *base;
+  char           *base_commit;
+  char           *base_version;
+  char          **base_extensions;
   char           *metadata;
   char           *metadata_platform;
   gboolean        separate_locales;
@@ -91,8 +95,13 @@ enum {
   PROP_BRANCH,
   PROP_RUNTIME,
   PROP_RUNTIME_VERSION,
+  PROP_RUNTIME_COMMIT,
   PROP_SDK,
   PROP_SDK_COMMIT,
+  PROP_BASE,
+  PROP_BASE_VERSION,
+  PROP_BASE_COMMIT,
+  PROP_BASE_EXTENSIONS,
   PROP_VAR,
   PROP_METADATA,
   PROP_METADATA_PLATFORM,
@@ -131,6 +140,9 @@ builder_manifest_finalize (GObject *object)
   g_free (self->runtime_version);
   g_free (self->sdk);
   g_free (self->sdk_commit);
+  g_free (self->base);
+  g_free (self->base_commit);
+  g_free (self->base_version);
   g_free (self->var);
   g_free (self->metadata);
   g_free (self->metadata_platform);
@@ -226,6 +238,10 @@ builder_manifest_get_property (GObject    *object,
       g_value_set_string (value, self->runtime);
       break;
 
+    case PROP_RUNTIME_COMMIT:
+      g_value_set_string (value, self->runtime_commit);
+      break;
+
     case PROP_RUNTIME_VERSION:
       g_value_set_string (value, self->runtime_version);
       break;
@@ -238,6 +254,22 @@ builder_manifest_get_property (GObject    *object,
       g_value_set_string (value, self->sdk_commit);
       break;
 
+    case PROP_BASE:
+      g_value_set_string (value, self->base);
+      break;
+
+    case PROP_BASE_COMMIT:
+      g_value_set_string (value, self->base_commit);
+      break;
+
+    case PROP_BASE_VERSION:
+      g_value_set_string (value, self->base_version);
+      break;
+
+    case PROP_BASE_EXTENSIONS:
+      g_value_set_boxed (value, self->base_extensions);
+      break;
+
     case PROP_VAR:
       g_value_set_string (value, self->var);
       break;
@@ -371,6 +403,11 @@ builder_manifest_set_property (GObject      *object,
       self->runtime = g_value_dup_string (value);
       break;
 
+    case PROP_RUNTIME_COMMIT:
+      g_free (self->runtime_commit);
+      self->runtime_commit = g_value_dup_string (value);
+      break;
+
     case PROP_RUNTIME_VERSION:
       g_free (self->runtime_version);
       self->runtime_version = g_value_dup_string (value);
@@ -386,6 +423,27 @@ builder_manifest_set_property (GObject      *object,
       self->sdk_commit = g_value_dup_string (value);
       break;
 
+    case PROP_BASE:
+      g_free (self->base);
+      self->base = g_value_dup_string (value);
+      break;
+
+    case PROP_BASE_COMMIT:
+      g_free (self->base_commit);
+      self->base_commit = g_value_dup_string (value);
+      break;
+
+    case PROP_BASE_VERSION:
+      g_free (self->base_version);
+      self->base_version = g_value_dup_string (value);
+      break;
+
+    case PROP_BASE_EXTENSIONS:
+      tmp = self->base_extensions;
+      self->base_extensions = g_strdupv (g_value_get_boxed (value));
+      g_strfreev (tmp);
+      break;
+
     case PROP_VAR:
       g_free (self->var);
       self->var = g_value_dup_string (value);
@@ -552,6 +610,13 @@ builder_manifest_class_init (BuilderManifestClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_RUNTIME_COMMIT,
+                                   g_param_spec_string ("runtime-commit",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_RUNTIME_VERSION,
                                    g_param_spec_string ("runtime-version",
@@ -573,6 +638,34 @@ builder_manifest_class_init (BuilderManifestClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_BASE,
+                                   g_param_spec_string ("base",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_BASE_COMMIT,
+                                   g_param_spec_string ("base-commit",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_BASE_VERSION,
+                                   g_param_spec_string ("base-version",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_BASE_EXTENSIONS,
+                                   g_param_spec_boxed ("base-extensions",
+                                                       "",
+                                                       "",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_VAR,
                                    g_param_spec_string ("var",
@@ -813,14 +906,19 @@ builder_manifest_deserialize_property (JsonSerializable *serializable,
                 {
                   const char *module_path = json_node_get_string (element_node);
                   g_autofree char *json = NULL;
+                  g_autoptr(GError) error = NULL;
 
-                  if (g_file_get_contents (module_path, &json, NULL, NULL))
+                  if (g_file_get_contents (module_path, &json, NULL, &error))
                     {
                       module = json_gobject_from_data (BUILDER_TYPE_MODULE,
-                                                       json, -1, NULL);
+                                                       json, -1, &error);
                       if (module)
                         builder_module_set_json_path (BUILDER_MODULE (module), module_path);
                     }
+                  if (error != NULL)
+                    {
+                      g_error ("Failed to load included manifest (%s): %s", module_path, error->message);
+                    }
                 }
               else if (JSON_NODE_HOLDS_OBJECT (element_node))
                 module = json_gobject_deserialize (BUILDER_TYPE_MODULE, element_node);
@@ -863,12 +961,58 @@ builder_manifest_get_id (BuilderManifest *self)
   return self->id;
 }
 
+/* Dashes are only valid in the last part of the app id, so
+   we replace them with underscore so we can suffix the id */
+static char *
+make_valid_id_prefix (const char *orig_id)
+{
+  char *id, *t;
+
+  id = g_strdup (orig_id);
+  t = strchr (id, '-');
+  while (t != NULL)
+    {
+      *t = '_';
+      t = strchr (t + 1, '-');
+    }
+
+  return id;
+}
+
+char *
+builder_manifest_get_locale_id (BuilderManifest *self)
+{
+  g_autofree char *id = make_valid_id_prefix (self->id);
+  return g_strdup_printf ("%s.Locale", id);
+}
+
+char *
+builder_manifest_get_debug_id (BuilderManifest *self)
+{
+  g_autofree char *id = make_valid_id_prefix (self->id);
+  return g_strdup_printf ("%s.Debug", id);
+}
+
 const char *
 builder_manifest_get_id_platform (BuilderManifest *self)
 {
   return self->id_platform;
 }
 
+char *
+builder_manifest_get_locale_id_platform (BuilderManifest *self)
+{
+  char *res = NULL;
+
+  if (self->id_platform != NULL)
+    {
+      g_autofree char *id = make_valid_id_prefix (self->id_platform);
+      res = g_strdup_printf ("%s.Locale", id);
+    }
+
+  return res;
+}
+
 BuilderOptions *
 builder_manifest_get_build_options (BuilderManifest *self)
 {
@@ -893,6 +1037,12 @@ builder_manifest_get_branch (BuilderManifest *self)
   return self->branch ? self->branch : "master";
 }
 
+static const char *
+builder_manifest_get_base_version (BuilderManifest *self)
+{
+  return self->base_version ? self->base_version : builder_manifest_get_branch (self);
+}
+
 static char *
 flatpak (GError **error,
          ...)
@@ -915,6 +1065,7 @@ flatpak (GError **error,
 
 gboolean
 builder_manifest_start (BuilderManifest *self,
+                        gboolean allow_missing_runtimes,
                         BuilderContext  *context,
                         GError         **error)
 {
@@ -933,18 +1084,27 @@ builder_manifest_start (BuilderManifest *self,
 
   self->sdk_commit = flatpak (NULL, "info", arch_option, "--show-commit", self->sdk,
                               builder_manifest_get_runtime_version (self), NULL);
-  if (self->sdk_commit == NULL)
+  if (!allow_missing_runtimes && self->sdk_commit == NULL)
     return flatpak_fail (error, "Unable to find sdk %s version %s",
                          self->sdk,
                          builder_manifest_get_runtime_version (self));
 
   self->runtime_commit = flatpak (NULL, "info", arch_option, "--show-commit", self->runtime,
                                   builder_manifest_get_runtime_version (self), NULL);
-  if (self->runtime_commit == NULL)
+  if (!allow_missing_runtimes && self->runtime_commit == NULL)
     return flatpak_fail (error, "Unable to find runtime %s version %s",
                          self->runtime,
                          builder_manifest_get_runtime_version (self));
 
+  if (self->base != NULL && *self->base != 0)
+    {
+      self->base_commit = flatpak (NULL, "info", arch_option, "--show-commit", self->base,
+                                   builder_manifest_get_base_version (self), NULL);
+      if (self->base_commit == NULL)
+        return flatpak_fail (error, "Unable to find app %s version %s",
+                             self->base, builder_manifest_get_base_version (self));
+    }
+
   if (!expand_modules (self->modules, &self->expanded_modules, names, error))
     return FALSE;
 
@@ -1011,6 +1171,19 @@ builder_manifest_init_app_dir (BuilderManifest *self,
     }
   if (self->var)
     g_ptr_array_add (args, g_strdup_printf ("--var=%s", self->var));
+
+  if (self->base)
+    {
+      g_ptr_array_add (args, g_strdup_printf ("--base=%s", self->base));
+      g_ptr_array_add (args, g_strdup_printf ("--base-version=%s", builder_manifest_get_base_version (self)));
+
+      for (i = 0; self->base_extensions != NULL && self->base_extensions[i] != NULL; i++)
+        {
+          const char *ext = self->base_extensions[i];
+          g_ptr_array_add (args, g_strdup_printf ("--base-extension=%s", ext));
+        }
+    }
+
   g_ptr_array_add (args, g_strdup_printf ("--arch=%s", builder_context_get_arch (context)));
   g_ptr_array_add (args, g_file_get_path (app_dir));
   g_ptr_array_add (args, g_strdup (self->id));
@@ -1064,6 +1237,10 @@ builder_manifest_checksum (BuilderManifest *self,
   builder_cache_checksum_strv (cache, self->sdk_extensions);
   builder_cache_checksum_boolean (cache, self->build_runtime);
   builder_cache_checksum_boolean (cache, self->separate_locales);
+  builder_cache_checksum_str (cache, self->base);
+  builder_cache_checksum_str (cache, self->base_version);
+  builder_cache_checksum_str (cache, self->base_commit);
+  builder_cache_checksum_strv (cache, self->base_extensions);
 
   if (self->build_options)
     builder_options_checksum (self->build_options, cache, context);
@@ -1152,12 +1329,20 @@ builder_manifest_download (BuilderManifest *self,
                            BuilderContext  *context,
                            GError         **error)
 {
+  const char *stop_at = builder_context_get_stop_at (context);
   GList *l;
 
   g_print ("Downloading sources\n");
   for (l = self->expanded_modules; l != NULL; l = l->next)
     {
       BuilderModule *m = l->data;
+      const char *name = builder_module_get_name (m);
+
+      if (stop_at != NULL && strcmp (name, stop_at) == 0)
+        {
+          g_print ("Stopping at module %s\n", stop_at);
+          return TRUE;
+        }
 
       if (!builder_module_download_sources (m, update_vcs, context, error))
         return FALSE;
@@ -1800,14 +1985,15 @@ builder_manifest_finish (BuilderManifest *self,
           g_autoptr(GFileOutputStream) output = NULL;
           g_autoptr(GFile) metadata_locale_file = NULL;
           g_autofree char *metadata_contents = NULL;
+          g_autofree char *locale_id = builder_manifest_get_locale_id (self);
 
           metadata_file = g_file_get_child (app_dir, "metadata");
 
           extension_contents = g_strdup_printf ("\n"
-                                                "[Extension %s.Locale]\n"
+                                                "[Extension %s]\n"
                                                 "directory=%s\n"
                                                 "autodelete=true\n",
-                                                self->id,
+                                                locale_id,
                                                 LOCALES_SEPARATE_DIR);
 
           output = g_file_append_to (metadata_file, G_FILE_CREATE_NONE, NULL, error);
@@ -1822,7 +2008,7 @@ builder_manifest_finish (BuilderManifest *self,
 
           metadata_locale_file = g_file_get_child (app_dir, "metadata.locale");
           metadata_contents = g_strdup_printf ("[Runtime]\n"
-                                               "name=%s.Locale\n", self->id);
+                                               "name=%s\n", locale_id);
           if (!g_file_replace_contents (metadata_locale_file,
                                         metadata_contents, strlen (metadata_contents),
                                         NULL, FALSE,
@@ -1839,16 +2025,17 @@ builder_manifest_finish (BuilderManifest *self,
           g_autofree char *metadata_contents = NULL;
           g_autofree char *extension_contents = NULL;
           g_autoptr(GFileOutputStream) output = NULL;
+          g_autofree char *debug_id = builder_manifest_get_debug_id (self);
 
           metadata_file = g_file_get_child (app_dir, "metadata");
           metadata_debuginfo_file = g_file_get_child (app_dir, "metadata.debuginfo");
 
           extension_contents = g_strdup_printf ("\n"
-                                                "[Extension %s.Debug]\n"
+                                                "[Extension %s]\n"
                                                 "directory=lib/debug\n"
                                                 "autodelete=true\n"
                                                 "no-autodownload=true\n",
-                                                self->id);
+                                                debug_id);
 
           output = g_file_append_to (metadata_file, G_FILE_CREATE_NONE, NULL, error);
           if (output == NULL)
@@ -1859,7 +2046,7 @@ builder_manifest_finish (BuilderManifest *self,
             return FALSE;
 
           metadata_contents = g_strdup_printf ("[Runtime]\n"
-                                               "name=%s.Debug\n", self->id);
+                                               "name=%s\n", debug_id);
           if (!g_file_replace_contents (metadata_debuginfo_file,
                                         metadata_contents, strlen (metadata_contents), NULL, FALSE,
                                         G_FILE_CREATE_REPLACE_DESTINATION,
@@ -2064,14 +2251,15 @@ builder_manifest_create_platform (BuilderManifest *self,
           g_autoptr(GFileOutputStream) output = NULL;
           g_autoptr(GFile) metadata_locale_file = NULL;
           g_autofree char *metadata_contents = NULL;
+          g_autofree char *locale_id = builder_manifest_get_locale_id_platform (self);
 
           metadata_file = g_file_get_child (app_dir, "metadata.platform");
 
           extension_contents = g_strdup_printf ("\n"
-                                                "[Extension %s.Locale]\n"
+                                                "[Extension %s]\n"
                                                 "directory=%s\n"
                                                 "autodelete=true\n",
-                                                self->id_platform,
+                                                locale_id,
                                                 LOCALES_SEPARATE_DIR);
 
           output = g_file_append_to (metadata_file, G_FILE_CREATE_NONE, NULL, error);
@@ -2086,7 +2274,7 @@ builder_manifest_create_platform (BuilderManifest *self,
 
           metadata_locale_file = g_file_get_child (app_dir, "metadata.platform.locale");
           metadata_contents = g_strdup_printf ("[Runtime]\n"
-                                               "name=%s.Locale\n", self->id_platform);
+                                               "name=%s\n", locale_id);
           if (!g_file_replace_contents (metadata_locale_file,
                                         metadata_contents, strlen (metadata_contents),
                                         NULL, FALSE,
@@ -2200,6 +2388,7 @@ builder_manifest_run (BuilderManifest *self,
   g_ptr_array_add (args, NULL);
 
   commandline = g_strjoinv (" ", (char **) args->pdata);
+  g_debug ("Running '%s'", commandline);
 
   if (execvp ((char *) args->pdata[0], (char **) args->pdata) == -1)
     {
