@@ -38,13 +38,23 @@ GType flatpak_dir_get_type (void);
 GType flatpak_deploy_get_type (void);
 
 #define FLATPAK_REF_GROUP "Flatpak Ref"
+#define FLATPAK_REF_VERSION_KEY "Version"
 #define FLATPAK_REF_URL_KEY "Url"
+#define FLATPAK_REF_RUNTIME_REPO_KEY "RuntimeRepo"
 #define FLATPAK_REF_TITLE_KEY "Title"
 #define FLATPAK_REF_GPGKEY_KEY "GPGKey"
 #define FLATPAK_REF_IS_RUNTIME_KEY "IsRuntime"
 #define FLATPAK_REF_NAME_KEY "Name"
 #define FLATPAK_REF_BRANCH_KEY "Branch"
 
+#define FLATPAK_REPO_GROUP "Flatpak Repo"
+#define FLATPAK_REPO_VERSION_KEY "Version"
+#define FLATPAK_REPO_URL_KEY "Url"
+#define FLATPAK_REPO_TITLE_KEY "Title"
+#define FLATPAK_REPO_DEFAULT_BRANCH_KEY "DefaultBranch"
+#define FLATPAK_REPO_GPGKEY_KEY "GPGKey"
+#define FLATPAK_REPO_NODEPS_KEY "NoDeps"
+
 typedef struct
 {
   char           *ref;
@@ -84,6 +94,19 @@ typedef enum {
 
 #define FLATPAK_HELPER_CONFIGURE_REMOTE_FLAGS_ALL (FLATPAK_HELPER_CONFIGURE_REMOTE_FLAGS_FORCE_REMOVE)
 
+typedef enum {
+  FLATPAK_PULL_FLAGS_NONE = 0,
+  FLATPAK_PULL_FLAGS_DOWNLOAD_EXTRA_DATA = 1 << 0,
+  FLATPAK_PULL_FLAGS_SIDELOAD_EXTRA_DATA = 1 << 1,
+} FlatpakPullFlags;
+
+typedef enum {
+  FLATPAK_DIR_STORAGE_TYPE_DEFAULT = 0,
+  FLATPAK_DIR_STORAGE_TYPE_HARD_DISK,
+  FLATPAK_DIR_STORAGE_TYPE_SDCARD,
+  FLATPAK_DIR_STORAGE_TYPE_MMC,
+} FlatpakDirStorageType;
+
 GQuark       flatpak_dir_error_quark (void);
 
 /**
@@ -98,8 +121,10 @@ GQuark       flatpak_dir_error_quark (void);
 #define FLATPAK_DEPLOY_DATA_GVARIANT_STRING "(ssasta{sv})"
 #define FLATPAK_DEPLOY_DATA_GVARIANT_FORMAT G_VARIANT_TYPE (FLATPAK_DEPLOY_DATA_GVARIANT_STRING)
 
-GFile *  flatpak_get_system_base_dir_location (void);
-GFile *  flatpak_get_user_base_dir_location (void);
+GPtrArray * flatpak_get_system_base_dir_locations (GCancellable *cancellable,
+                                                   GError      **error);
+GFile *     flatpak_get_system_default_base_dir_location (void);
+GFile *     flatpak_get_user_base_dir_location (void);
 
 GKeyFile *     flatpak_load_override_keyfile (const char *app_id,
                                               gboolean    user,
@@ -116,6 +141,7 @@ const char *        flatpak_deploy_data_get_origin (GVariant *deploy_data);
 const char *        flatpak_deploy_data_get_commit (GVariant *deploy_data);
 const char **       flatpak_deploy_data_get_subpaths (GVariant *deploy_data);
 guint64             flatpak_deploy_data_get_installed_size (GVariant *deploy_data);
+const char *        flatpak_deploy_data_get_alt_id (GVariant *deploy_data);
 
 GFile *        flatpak_deploy_get_dir (FlatpakDeploy *deploy);
 GFile *        flatpak_deploy_get_files (FlatpakDeploy *deploy);
@@ -125,16 +151,29 @@ GKeyFile *     flatpak_deploy_get_metadata (FlatpakDeploy *deploy);
 FlatpakDir *  flatpak_dir_new (GFile   *basedir,
                                gboolean user);
 FlatpakDir *  flatpak_dir_clone (FlatpakDir *self);
-FlatpakDir  *flatpak_dir_get (gboolean user);
-FlatpakDir  *flatpak_dir_get_system (void);
 FlatpakDir  *flatpak_dir_get_user (void);
+FlatpakDir  *flatpak_dir_get_system_default (void);
+GPtrArray   *flatpak_dir_get_system_list (GCancellable *cancellable,
+                                          GError      **error);
+FlatpakDir  *flatpak_dir_get_system_by_id (const char   *id,
+                                           GCancellable *cancellable,
+                                           GError      **error);
 gboolean    flatpak_dir_is_user (FlatpakDir *self);
 void        flatpak_dir_set_no_system_helper (FlatpakDir *self,
                                               gboolean    no_system_helper);
 GFile *     flatpak_dir_get_path (FlatpakDir *self);
 GFile *     flatpak_dir_get_changed_path (FlatpakDir *self);
+const char *flatpak_dir_get_id (FlatpakDir *self);
+const char *flatpak_dir_get_display_name (FlatpakDir *self);
+char *      flatpak_dir_get_name (FlatpakDir *self);
+gint        flatpak_dir_get_priority (FlatpakDir *self);
+FlatpakDirStorageType flatpak_dir_get_storage_type (FlatpakDir *self);
 GFile *     flatpak_dir_get_deploy_dir (FlatpakDir *self,
                                         const char *ref);
+GFile *     flatpak_dir_get_unmaintained_extension_dir (FlatpakDir *self,
+                                                        const char *name,
+                                                        const char *arch,
+                                                        const char *branch);
 GVariant *  flatpak_dir_get_deploy_data (FlatpakDir   *dir,
                                          const char   *ref,
                                          GCancellable *cancellable,
@@ -153,14 +192,27 @@ GFile *     flatpak_dir_get_if_deployed (FlatpakDir   *self,
                                          const char   *ref,
                                          const char   *checksum,
                                          GCancellable *cancellable);
+GFile *     flatpak_dir_get_unmaintained_extension_dir_if_exists (FlatpakDir *self,
+                                                                  const char *name,
+                                                                  const char *arch,
+                                                                  const char *branch,
+                                                                  GCancellable *cancellable);
+
+gboolean    flatpak_dir_remote_has_ref (FlatpakDir   *self,
+                                        const char   *remote,
+                                        const char   *ref);
+char **     flatpak_dir_search_for_dependency (FlatpakDir   *self,
+                                               const char   *runtime_ref,
+                                               GCancellable *cancellable,
+                                               GError      **error);
 char *      flatpak_dir_find_remote_ref (FlatpakDir   *self,
                                          const char   *remote,
                                          const char   *name,
                                          const char   *opt_branch,
+                                         const char   *opt_default_branch,
                                          const char   *opt_arch,
-                                         gboolean      app,
-                                         gboolean      runtime,
-                                         gboolean     *is_app,
+                                         FlatpakKinds  kinds,
+                                         FlatpakKinds *out_kind,
                                          GCancellable *cancellable,
                                          GError      **error);
 char **     flatpak_dir_find_remote_refs (FlatpakDir   *self,
@@ -168,25 +220,22 @@ char **     flatpak_dir_find_remote_refs (FlatpakDir   *self,
                                           const char   *name,
                                           const char   *opt_branch,
                                           const char   *opt_arch,
-                                          gboolean      app,
-                                          gboolean      runtime,
+                                          FlatpakKinds  kinds,
                                           GCancellable *cancellable,
                                           GError      **error);
-char *      flatpak_dir_find_installed_ref (FlatpakDir *self,
-                                            const char *opt_name,
-                                            const char *opt_branch,
-                                            const char *opt_arch,
-                                            gboolean    app,
-                                            gboolean    runtime,
-                                            gboolean   *is_app,
-                                            GError    **error);
-char **     flatpak_dir_find_installed_refs (FlatpakDir *self,
-                                             const char *opt_name,
-                                             const char *opt_branch,
-                                             const char *opt_arch,
-                                             gboolean    app,
-                                             gboolean    runtime,
-                                             GError    **error);
+char *      flatpak_dir_find_installed_ref (FlatpakDir  *self,
+                                            const char  *opt_name,
+                                            const char  *opt_branch,
+                                            const char  *opt_arch,
+                                            FlatpakKinds kinds,
+                                            FlatpakKinds *out_kind,
+                                            GError     **error);
+char **     flatpak_dir_find_installed_refs (FlatpakDir  *self,
+                                             const char  *opt_name,
+                                             const char  *opt_branch,
+                                             const char  *opt_arch,
+                                             FlatpakKinds kinds,
+                                             GError     **error);
 FlatpakDeploy *flatpak_dir_load_deployed (FlatpakDir   *self,
                                           const char   *ref,
                                           const char   *checksum,
@@ -203,6 +252,9 @@ gboolean    flatpak_dir_ensure_path (FlatpakDir   *self,
 gboolean    flatpak_dir_use_child_repo (FlatpakDir *self);
 gboolean    flatpak_dir_ensure_system_child_repo (FlatpakDir *self,
                                                   GError    **error);
+gboolean    flatpak_dir_recreate_repo (FlatpakDir   *self,
+                                       GCancellable *cancellable,
+                                       GError      **error);
 gboolean    flatpak_dir_ensure_repo (FlatpakDir   *self,
                                      GCancellable *cancellable,
                                      GError      **error);
@@ -228,8 +280,10 @@ gboolean    flatpak_dir_update_appstream (FlatpakDir          *self,
 gboolean    flatpak_dir_pull (FlatpakDir          *self,
                               const char          *repository,
                               const char          *ref,
+                              const char          *opt_rev,
                               const char         **subpaths,
                               OstreeRepo          *repo,
+                              FlatpakPullFlags     flatpak_flags,
                               OstreeRepoPullFlags  flags,
                               OstreeAsyncProgress *progress,
                               GCancellable        *cancellable,
@@ -256,6 +310,7 @@ gboolean    flatpak_dir_list_refs (FlatpakDir   *self,
 char *      flatpak_dir_read_latest (FlatpakDir   *self,
                                      const char   *remote,
                                      const char   *ref,
+                                     char        **out_alt_id,
                                      GCancellable *cancellable,
                                      GError      **error);
 char *      flatpak_dir_read_active (FlatpakDir   *self,
@@ -315,9 +370,17 @@ gboolean   flatpak_dir_install (FlatpakDir          *self,
                                 OstreeAsyncProgress *progress,
                                 GCancellable        *cancellable,
                                 GError             **error);
+char *flatpak_dir_ensure_bundle_remote (FlatpakDir          *self,
+                                        GFile               *file,
+                                        GBytes              *extra_gpg_data,
+                                        char               **out_ref,
+                                        char               **out_metadata,
+                                        gboolean            *out_created_remote,
+                                        GCancellable        *cancellable,
+                                        GError             **error);
 gboolean flatpak_dir_install_bundle (FlatpakDir          *self,
                                      GFile               *file,
-                                     GBytes              *extra_gpg_data,
+                                     const char          *remote,
                                      char               **out_ref,
                                      GCancellable        *cancellable,
                                      GError             **error);
@@ -381,23 +444,44 @@ gboolean    flatpak_dir_collect_deployed_refs (FlatpakDir   *self,
                                                GHashTable   *hash,
                                                GCancellable *cancellable,
                                                GError      **error);
+gboolean    flatpak_dir_collect_unmaintained_refs (FlatpakDir   *self,
+                                                   const char   *name_prefix,
+                                                   const char   *arch,
+                                                   const char   *branch,
+                                                   GHashTable   *hash,
+                                                   GCancellable *cancellable,
+                                                   GError      **error);
 char      *flatpak_dir_create_origin_remote (FlatpakDir   *self,
                                              const char   *url,
                                              const char   *id,
                                              const char   *title,
                                              const char   *main_ref,
+                                             const char   *oci_uri,
+                                             const char   *oci_tag,
                                              GBytes       *gpg_data,
                                              GCancellable *cancellable,
                                              GError      **error);
 gboolean   flatpak_dir_create_remote_for_ref_file (FlatpakDir   *self,
                                                    GBytes  *data,
+                                                   const char *default_arch,
                                                    char   **remote_name_out,
                                                    char   **ref_out,
                                                    GError **error);
+GKeyFile * flatpak_dir_parse_repofile (FlatpakDir   *self,
+                                       const char   *remote_name,
+                                       GBytes       *data,
+                                       GBytes      **gpg_data_out,
+                                       GCancellable *cancellable,
+                                       GError      **error);
 
+char      *flatpak_dir_find_remote_by_uri (FlatpakDir   *self,
+                                           const char   *uri);
 char     **flatpak_dir_list_remotes (FlatpakDir   *self,
                                      GCancellable *cancellable,
                                      GError      **error);
+char     **flatpak_dir_list_enumerated_remotes (FlatpakDir   *self,
+                                                GCancellable *cancellable,
+                                                GError      **error);
 gboolean   flatpak_dir_modify_remote (FlatpakDir   *self,
                                       const char   *remote_name,
                                       GKeyFile     *config,
@@ -411,10 +495,20 @@ gboolean   flatpak_dir_remove_remote (FlatpakDir   *self,
                                       GError      **error);
 char      *flatpak_dir_get_remote_title (FlatpakDir *self,
                                          const char *remote_name);
+char      *flatpak_dir_get_remote_main_ref (FlatpakDir *self,
+                                            const char *remote_name);
+char      *flatpak_dir_get_remote_oci_uri (FlatpakDir *self,
+                                           const char *remote_name);
+char      *flatpak_dir_get_remote_oci_tag (FlatpakDir *self,
+                                           const char *remote_name);
+char      *flatpak_dir_get_remote_default_branch (FlatpakDir *self,
+                                                  const char *remote_name);
 int        flatpak_dir_get_remote_prio (FlatpakDir *self,
                                         const char *remote_name);
 gboolean   flatpak_dir_get_remote_noenumerate (FlatpakDir *self,
                                                const char *remote_name);
+gboolean   flatpak_dir_get_remote_nodeps (FlatpakDir *self,
+                                          const char *remote_name);
 gboolean   flatpak_dir_get_remote_disabled (FlatpakDir *self,
                                             const char *remote_name);
 gboolean   flatpak_dir_list_remote_refs (FlatpakDir   *self,
@@ -426,6 +520,14 @@ char *   flatpak_dir_fetch_remote_title (FlatpakDir   *self,
                                          const char   *remote,
                                          GCancellable *cancellable,
                                          GError      **error);
+char *   flatpak_dir_fetch_remote_default_branch (FlatpakDir   *self,
+                                                  const char   *remote,
+                                                  GCancellable *cancellable,
+                                                  GError      **error);
+gboolean flatpak_dir_update_remote_configuration (FlatpakDir   *self,
+                                                  const char   *remote,
+                                                  GCancellable *cancellable,
+                                                  GError      **error);
 gboolean flatpak_dir_fetch_ref_cache (FlatpakDir   *self,
                                       const char   *remote_name,
                                       const char   *ref,
