@@ -24,18 +24,28 @@
 #include <string.h>
 
 #include "libglnx/libglnx.h"
+#include <flatpak-common-types.h>
 #include <gio/gio.h>
 #include <gio/gunixfdlist.h>
 #include <libsoup/soup.h>
 #include "flatpak-dbus.h"
-#include "flatpak-dir.h"
 #include <ostree.h>
+#include <json-glib/json-glib.h>
 
 typedef enum {
   FLATPAK_HOST_COMMAND_FLAGS_CLEAR_ENV = 1 << 0,
 } FlatpakHostCommandFlags;
 
 
+/* https://bugzilla.gnome.org/show_bug.cgi?id=766370 */
+#if !GLIB_CHECK_VERSION(2, 49, 3)
+#define FLATPAK_VARIANT_BUILDER_INITIALIZER {{0,}}
+#define FLATPAK_VARIANT_DICT_INITIALIZER {{0,}}
+#else
+#define FLATPAK_VARIANT_BUILDER_INITIALIZER {{{0,}}}
+#define FLATPAK_VARIANT_DICT_INITIALIZER {{{0,}}}
+#endif
+
 gboolean flatpak_fail (GError    **error,
                        const char *format,
                        ...);
@@ -43,6 +53,9 @@ gboolean flatpak_fail (GError    **error,
 gint flatpak_strcmp0_ptr (gconstpointer a,
                           gconstpointer b);
 
+gboolean  flatpak_has_path_prefix (const char *str,
+                                   const char *prefix);
+
 const char * flatpak_path_match_prefix (const char *pattern,
                                         const char *path);
 
@@ -65,6 +78,10 @@ gboolean flatpak_variant_save (GFile        *dest,
                                GVariant     *variant,
                                GCancellable *cancellable,
                                GError      **error);
+GVariant * flatpak_gvariant_new_empty_string_dict (void);
+void    flatpak_variant_builder_init_from_variant (GVariantBuilder *builder,
+                                                   const char      *type,
+                                                   GVariant        *variant);
 gboolean flatpak_variant_bsearch_str (GVariant   *array,
                                       const char *str,
                                       int        *out_pos);
@@ -86,10 +103,25 @@ gboolean flatpak_is_valid_branch (const char *string,
 char **flatpak_decompose_ref (const char *ref,
                               GError    **error);
 
-gboolean flatpak_split_partial_ref_arg (char *partial_ref,
-                                        char **inout_arch,
-                                        char **inout_branch,
-                                        GError    **error);
+FlatpakKinds flatpak_kinds_from_bools (gboolean app, gboolean runtime);
+
+gboolean flatpak_split_partial_ref_arg (const char   *partial_ref,
+                                        FlatpakKinds  default_kinds,
+                                        const char   *default_arch,
+                                        const char   *default_branch,
+                                        FlatpakKinds *out_kinds,
+                                        char        **out_id,
+                                        char        **out_arch,
+                                        char        **out_branch,
+                                        GError      **error);
+gboolean flatpak_split_partial_ref_arg_novalidate (const char   *partial_ref,
+                                                   FlatpakKinds  default_kinds,
+                                                   const char   *default_arch,
+                                                   const char   *default_branch,
+                                                   FlatpakKinds *out_kinds,
+                                                   char        **out_id,
+                                                   char        **out_arch,
+                                                   char        **out_branch);
 
 char * flatpak_compose_ref (gboolean    app,
                             const char *name,
@@ -106,9 +138,17 @@ char * flatpak_build_runtime_ref (const char *runtime,
 char * flatpak_build_app_ref (const char *app,
                               const char *branch,
                               const char *arch);
+GFile *flatpak_find_deploy_dir_for_ref (const char   *ref,
+                                        FlatpakDir **dir_out,
+                                        GCancellable *cancellable,
+                                        GError      **error);
 GFile * flatpak_find_files_dir_for_ref (const char   *ref,
                                         GCancellable *cancellable,
                                         GError      **error);
+GFile * flatpak_find_unmaintained_extension_dir_if_exists (const char   *name,
+                                                           const char   *arch,
+                                                           const char   *branch,
+                                                           GCancellable *cancellable);
 FlatpakDeploy * flatpak_find_deploy_for_ref (const char   *ref,
                                              GCancellable *cancellable,
                                              GError      **error);
@@ -118,6 +158,11 @@ char ** flatpak_list_deployed_refs (const char   *type,
                                     const char   *arch,
                                     GCancellable *cancellable,
                                     GError      **error);
+char ** flatpak_list_unmaintained_refs (const char   *name_prefix,
+                                        const char   *branch,
+                                        const char   *arch,
+                                        GCancellable *cancellable,
+                                        GError      **error);
 
 gboolean flatpak_overlay_symlink_tree (GFile        *source,
                                        GFile        *destination,
@@ -203,6 +248,9 @@ FlatpakTablePrinter *flatpak_table_printer_new (void);
 void                flatpak_table_printer_free (FlatpakTablePrinter *printer);
 void                flatpak_table_printer_add_column (FlatpakTablePrinter *printer,
                                                       const char          *text);
+void                flatpak_table_printer_add_column_len (FlatpakTablePrinter *printer,
+                                                          const char          *text,
+                                                          gsize                len);
 void                flatpak_table_printer_append_with_comma (FlatpakTablePrinter *printer,
                                                              const char          *text);
 void                flatpak_table_printer_finish_row (FlatpakTablePrinter *printer);
@@ -211,6 +259,9 @@ void                flatpak_table_printer_print (FlatpakTablePrinter *printer);
 gboolean flatpak_repo_set_title (OstreeRepo *repo,
                                  const char *title,
                                  GError    **error);
+gboolean flatpak_repo_set_default_branch (OstreeRepo *repo,
+                                          const char *branch,
+                                          GError    **error);
 gboolean flatpak_repo_update (OstreeRepo   *repo,
                               const char  **gpg_key_ids,
                               const char   *gpg_homedir,
@@ -222,7 +273,19 @@ gboolean flatpak_repo_collect_sizes (OstreeRepo   *repo,
                                      guint64      *download_size,
                                      GCancellable *cancellable,
                                      GError      **error);
-
+GVariant *flatpak_commit_get_extra_data_sources (GVariant *commitv,
+                                                 GError  **error);
+GVariant *flatpak_repo_get_extra_data_sources (OstreeRepo   *repo,
+                                               const char   *rev,
+                                               GCancellable *cancellable,
+                                               GError      **error);
+void flatpak_repo_parse_extra_data_sources (GVariant *extra_data_sources,
+                                            int index,
+                                            const char **name,
+                                            guint64 *download_size,
+                                            guint64 *installed_size,
+                                            const guchar **sha256,
+                                            const char **uri);
 gboolean flatpak_mtree_create_root (OstreeRepo        *repo,
                                     OstreeMutableTree *mtree,
                                     GCancellable      *cancellable,
@@ -232,6 +295,8 @@ GVariant * flatpak_bundle_load (GFile   *file,
                                 char   **commit,
                                 char   **ref,
                                 char   **origin,
+                                char   **runtime_repo,
+                                char   **app_metadata,
                                 guint64 *installed_size,
                                 GBytes **gpg_keys,
                                 GError **error);
@@ -244,6 +309,13 @@ gboolean flatpak_pull_from_bundle (OstreeRepo   *repo,
                                    GCancellable *cancellable,
                                    GError      **error);
 
+char * flatpak_pull_from_oci (OstreeRepo   *repo,
+                              FlatpakOciRegistry *registry,
+                              const char *digest,
+                              FlatpakOciManifest *manifest,
+                              const char *ref,
+                              GCancellable *cancellable,
+                              GError      **error);
 
 typedef struct
 {
@@ -253,6 +325,7 @@ typedef struct
   char *directory;
   char *files_path;
   gboolean needs_tmpfs;
+  gboolean is_unmaintained;
 } FlatpakExtension;
 
 void flatpak_extension_free (FlatpakExtension *extension);
@@ -306,6 +379,12 @@ gboolean flatpak_rm_rf (GFile         *dir,
                         GCancellable  *cancellable,
                         GError       **error);
 
+char * flatpak_readlink (const char *path,
+                         GError       **error);
+char * flatpak_resolve_link (const char *path,
+                             GError **error);
+char * flatpak_canonicalize_filename (const char *path);
+
 gboolean flatpak_file_rename (GFile *from,
                               GFile *to,
                               GCancellable  *cancellable,
@@ -374,9 +453,21 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC (OstreeRepoCommitModifier, ostree_repo_commit_modi
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupSession, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupMessage, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupRequest, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupRequestHTTP, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupURI, soup_uri_free)
 #endif
 
+#if !JSON_CHECK_VERSION(1,1,2)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonArray, json_array_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonBuilder, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonGenerator, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonNode, json_node_free)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonObject, json_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonParser, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonPath, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (JsonReader, g_object_unref)
+#endif
+
 #if !GLIB_CHECK_VERSION(2, 43, 4)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GUnixFDList, g_object_unref)
 #endif
@@ -446,6 +537,27 @@ gboolean flatpak_allocate_tmpdir (int           tmpdir_dfd,
                                   GError      **error);
 
 
+gboolean flatpak_yes_no_prompt (const char *prompt, ...);
+long flatpak_number_prompt (int min, int max, const char *prompt, ...);
+
+typedef void (*FlatpakLoadUriProgress) (guint64 downloaded_bytes,
+                                        gpointer user_data);
+
+SoupSession * flatpak_create_soup_session (const char *user_agent);
+GBytes * flatpak_load_http_uri (SoupSession *soup_session,
+                                const char   *uri,
+                                FlatpakLoadUriProgress progress,
+                                gpointer      user_data,
+                                GCancellable *cancellable,
+                                GError      **error);
+gboolean flatpak_download_http_uri (SoupSession *soup_session,
+                                    const char   *uri,
+                                    GOutputStream *out,
+                                    FlatpakLoadUriProgress progress,
+                                    gpointer      user_data,
+                                    GCancellable *cancellable,
+                                    GError      **error);
+
 typedef struct {
   char *shell_cur;
   char *cur;
@@ -468,6 +580,11 @@ void               flatpak_complete_word    (FlatpakCompletion *completion,
                                              ...);
 void               flatpak_complete_ref     (FlatpakCompletion *completion,
                                              OstreeRepo        *repo);
+void               flatpak_complete_partial_ref (FlatpakCompletion *completion,
+                                                 FlatpakKinds kinds,
+                                                 const char *only_arch,
+                                                 FlatpakDir *dir,
+                                                 const char *remote);
 void               flatpak_complete_file    (FlatpakCompletion *completion);
 void               flatpak_complete_dir     (FlatpakCompletion *completion);
 void               flatpak_complete_options (FlatpakCompletion *completion,
