@@ -701,7 +701,7 @@ xdp_inode_create_file (XdpInode   *dir,
   inode->dir_fd = glnx_steal_fd (&dir_fd);
   inode->fd = glnx_steal_fd (&fd);
   inode->trunc_fd = glnx_steal_fd (&trunc_fd);
-  if (inode->trunc_fd != -1 && truncate)
+  if (inode->trunc_fd != -1 && (truncate || exclusive))
     {
       inode->truncated = TRUE;
       g_free (inode->backing_filename);
@@ -1133,7 +1133,7 @@ static void
 xdp_fuse_readdir (fuse_req_t req, fuse_ino_t ino, size_t size,
                   off_t off, struct fuse_file_info *fi)
 {
-  struct dirbuf *b = (struct dirbuf *) (fi->fh);
+  struct dirbuf *b = (struct dirbuf *) (gsize) (fi->fh);
 
   reply_buf_limited (req, b->p, b->size, off, size);
 }
@@ -1242,7 +1242,7 @@ xdp_fuse_opendir (fuse_req_t             req,
       if (fuse_reply_open (req, fi) == -ENOENT)
         {
           g_free (b.p);
-          g_free ((gpointer) (fi->fh));
+          g_free ((gpointer) (gsize) (fi->fh));
         }
     }
 }
@@ -1252,7 +1252,7 @@ xdp_fuse_releasedir (fuse_req_t             req,
                      fuse_ino_t             ino,
                      struct fuse_file_info *fi)
 {
-  struct dirbuf *b = (struct dirbuf *) (fi->fh);
+  struct dirbuf *b = (struct dirbuf *) (gsize) (fi->fh);
 
   g_free (b->p);
   g_free (b);
@@ -1623,8 +1623,8 @@ xdp_fuse_create (fuse_req_t             req,
 
   inode = xdp_inode_create_file (parent_inode, filename,
                                  mode,
-                                 (fi->flags | O_TRUNC) != 0,
-                                 (fi->flags | O_EXCL) != 0);
+                                 (fi->flags & O_TRUNC) != 0,
+                                 (fi->flags & O_EXCL) != 0);
   if (inode == NULL)
     {
       fuse_reply_err (req, errno);
@@ -1693,7 +1693,7 @@ xdp_fuse_read (fuse_req_t             req,
                off_t                  off,
                struct fuse_file_info *fi)
 {
-  XdpFile *file = (gpointer) fi->fh;
+  XdpFile *file = (gpointer) (gsize) fi->fh;
   XdpInode *inode = file->inode;
   struct fuse_bufvec bufv = FUSE_BUFVEC_INIT (size);
   int fd;
@@ -1729,7 +1729,7 @@ xdp_fuse_release (fuse_req_t             req,
                   fuse_ino_t             ino,
                   struct fuse_file_info *fi)
 {
-  XdpFile *file = (gpointer) fi->fh;
+  XdpFile *file = (gpointer) (gsize) fi->fh;
 
   g_debug ("xdp_fuse_release %lx (fi=%p)", ino, fi);
 
@@ -1744,7 +1744,7 @@ truncateat (int dir_fd, const char *filename, int size)
   int errsv, res;
 
   fd = openat (dir_fd, filename, O_RDWR);
-  if (fd != -1)
+  if (fd == -1)
     return -1;
 
   res = ftruncate (fd, size);
@@ -1876,7 +1876,7 @@ xdp_fuse_setattr (fuse_req_t             req,
 
   if (res != 0)
     {
-      fuse_reply_err (req, ENOSYS);
+      fuse_reply_err (req, res);
     }
   else
     {
@@ -1895,7 +1895,7 @@ xdp_fuse_write (fuse_req_t             req,
                 off_t                  off,
                 struct fuse_file_info *fi)
 {
-  XdpFile *file = (gpointer) fi->fh;
+  XdpFile *file = (gpointer) (gsize) fi->fh;
   XdpInode *inode = file->inode;
   int fd;
   int res;
@@ -1928,7 +1928,7 @@ xdp_fuse_write_buf (fuse_req_t             req,
                     off_t                  off,
                     struct fuse_file_info *fi)
 {
-  XdpFile *file = (gpointer) fi->fh;
+  XdpFile *file = (gpointer) (gsize) fi->fh;
   struct fuse_bufvec dst = FUSE_BUFVEC_INIT (fuse_buf_size (bufv));
   XdpInode *inode = file->inode;
   int fd;
@@ -1941,6 +1941,7 @@ xdp_fuse_write_buf (fuse_req_t             req,
   fd = xdp_inode_locked_get_write_fd (inode);
   if (fd == -1)
     {
+      g_debug ("xdp_fuse_write_buf <- error %s", strerror (errno));
       fuse_reply_err (req, errno);
     }
   else
@@ -2134,37 +2135,59 @@ xdp_fuse_access (fuse_req_t req, fuse_ino_t ino, int mask)
 
   if (inode->type != XDP_INODE_DOC_FILE)
     {
-      g_debug ("xdp_fuse_access <- not file ENOSYS");
-      fuse_reply_err (req, ENOSYS);
-      return;
-    }
+      int dir_mask = 0;
 
-  entry = xdp_lookup_doc (inode->doc_id);
-  if (entry == NULL ||
-      !app_can_see_doc (entry, inode->app_id))
-    {
-      g_debug ("xdp_fuse_access <- no entry error ENOENT");
-      fuse_reply_err (req, ENOENT);
-      return;
-    }
+      switch (inode->type)
+        {
+        case XDP_INODE_ROOT:
+        case XDP_INODE_BY_APP:
+        case XDP_INODE_APP_DIR:
+          dir_mask = R_OK | X_OK;
+          break;
+        case XDP_INODE_APP_DOC_DIR:
+        case XDP_INODE_DOC_DIR:
+          dir_mask = R_OK | X_OK | W_OK;
+          break;
 
-  if (mask == F_OK)
-    {
-      if (!app_can_see_doc (entry, inode->app_id))
+        default:
+          g_assert_not_reached ();
+        }
+
+      if (mask != F_OK && ((mask & dir_mask) != mask))
         {
           fuse_reply_err (req, EACCES);
           return;
         }
     }
-  else
+  else /* A file */
     {
-      if (((mask & R_OK) && !app_can_see_doc (entry, inode->app_id)) ||
-          ((mask & W_OK) && !app_can_write_doc (entry, inode->app_id)) ||
-           (mask & X_OK))
+      entry = xdp_lookup_doc (inode->doc_id);
+      if (entry == NULL ||
+          !app_can_see_doc (entry, inode->app_id))
         {
-          fuse_reply_err (req, EACCES);
+          g_debug ("xdp_fuse_access <- no entry error ENOENT");
+          fuse_reply_err (req, ENOENT);
           return;
         }
+
+      if (mask == F_OK)
+        {
+          if (!app_can_see_doc (entry, inode->app_id))
+            {
+              fuse_reply_err (req, EACCES);
+              return;
+            }
+        }
+      else
+        {
+          if (((mask & R_OK) && !app_can_see_doc (entry, inode->app_id)) ||
+              ((mask & W_OK) && !app_can_write_doc (entry, inode->app_id)) ||
+              (mask & X_OK))
+            {
+              fuse_reply_err (req, EACCES);
+              return;
+            }
+        }
     }
 
   fuse_reply_err (req, 0);
